;==============================================================================
;*// File    : GETDRINF.ASM
;*// Author  : Eric Woodruff  CIS ID: 72134,1150
;*// Updated : Wed 12/21/94 20:09:47
;*// Note    : Copyright 1993-94, Eric Woodruff, All Rights Reserved.
;*// Compiler: TASM 3.1 to 4.0
;*//
;*// tasm /m2 /ml /q /w2 /zn [/d__l__] getdrinf.asm
;*//
;*// This routine returns information about the drives found on the system.
;*// The parameters passed are an address to a string (must be at least 27
;*// bytes long!) and Drive Type(s) to scan for.  The routine stores drive
;*// letters for the specified type(s) into the drive string and NULL
;*// terminates it.
;*//

IFDEF __l__
    DISPLAY "GETDRINF.ASM compiled for LARGE memory model"
    MODEL LARGE, C
ELSE
    DISPLAY "GETDRINF.ASM compiled for SMALL memory model"
    MODEL SMALL, C
ENDIF

        CODESEG

;=============================================================================
; Equates.
IFDEF __l__
    DriveString     equ     bp + 6      ; Drive list string address.
    DriveType       equ     bp + 10     ; Defines what drives to get.
ELSE
    DriveString     equ     bp + 4
    DriveType       equ     bp + 6
ENDIF

; Drive types to get.
drRemoveableDrives  equ     01h
drFixedDrives       equ     02h
drRAMDrives         equ     04h
drCDROMDrives       equ     08h
drNetworkDrives     equ     10h
drSubstitutedDrives equ     20h
drAllValidDrives    equ     3Fh         ; Get all valid drives.

; Drive type bit masks.
drNetDriveMask      equ     1000h
drSubstDriveMask    equ     8000h
drNetOrSubstMask    equ     9000h

;=============================================================================
        PUBLIC  C  GetDriveInfo

GetDriveInfo  PROC  C

        push    bp
        mov     bp,sp
        push    di

        cld
IFDEF __l__
        les     di, dword ptr [DriveString]
ELSE
        push    ds
        pop     es
        mov     di, word ptr [DriveString]
ENDIF
        xor     bx, bx                  ; Drive to check (1=A:, 2=B:, etc)

DriveLoop:
        inc     bl
        mov     ax, 4408h               ; First, is this drive removeable?
        int     21h
        jc      short CheckCD           ; Carry set = invalid, but CD-ROMS
                                        ; and certain network drives show
                                        ; up as invalid when they are not.

        or      ax, ax                  ; Removeable?
        jnz     short CheckCD           ; No.

        test    byte ptr [DriveType], drRemoveableDrives  ; Find removeables?
        jnz     short Store
        jmp     short NextDrive

CheckCD:
        mov     ax, 4409h               ; Is the drive local or remote?
        int     21h
        jc      short NextDrive         ; This drive letter really is invalid.

        push    dx                      ; CD-ROM drives also have the
        push    bx                      ; remote bit set in DX so
        mov     ax, 150Bh               ; eliminate them first.
        mov     cx, bx
        xor     bx, bx
        dec     cl                      ; For CD check, 0=A:, 1=B:, etc.
        int     2Fh
        cmp     bx, 0ADADh
        pop     bx
        pop     dx
        jne     short ChkFixed          ; No CD-ROM driver support.

        or      ax, ax                  ; Is this drive a CD-ROM?
        jz      short ChkFixed

        test    byte ptr [DriveType], drCDROMDrives   ; CD-ROM drives?
        jnz     short Store
        jmp     short NextDrive

ChkFixed:
        and     dx, drNetOrSubstMask    ; Network or substituted?
        jnz     short ChkNetOrSubst     ; Yes.

        push    ds
        push    bx
        mov     ah, 32h                 ; Must be a hard drive or RAM drive.
        mov     dl, bl                  ; Find out which type it is.
        int     21h

        cmp     byte ptr DS:[bx+8], 1   ; RAM drives usually only have
        pop     bx                      ; 1 copy of the FAT.  However,
        pop     ds                      ; this may not always be true but
        jne     NotRAMDrive             ; it is a fairly reliable test.

        test    byte ptr [DriveType], drRAMDrives
        jnz     short Store
        jmp     short NextDrive

NotRAMDrive:
        test    byte ptr [DriveType], drFixedDrives
        jnz     short Store
        jmp     short NextDrive

ChkNetOrSubst:
        test    byte ptr [DriveType], drNetworkDrives
        jz      short ChkSubst

        test    dx, drNetDriveMask      ; Network drive?
        jnz     short Store

ChkSubst:
        test    byte ptr [DriveType], drSubstitutedDrives
        jz      short NextDrive

        test    dx, drNetDriveMask      ; Must be looking for substituted ones,
        jnz     short NextDrive         ; so ignore network drives (they have
                                        ; the substitute bit set too).

        test    dx, drSubstDriveMask
        jz      short NextDrive

Store:
        mov     al, bl
        add     al, 64                  ; Store drive letter in the list.
        stosb

NextDrive:
        cmp     bl, 26
        je      short Done
        jmp     DriveLoop

Done:
        xor     al, al
        stosb

        pop     di
        pop     bp
        ret

GetDriveInfo    endp

        end
